package cytoPrefuse;

import giny.model.Edge;
import giny.model.Node;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import org.jfree.ui.BevelArrowIcon;

import cytoscape.Cytoscape;
import cytoscape.data.CyAttributesUtils;

public class CreateViewDialog extends JPanel {

	
	private static final long serialVersionUID = 1L;

	private JTable attributeOrderTable;
	private LocalizationTableModel localizationTableModel;
	private JTable edgeInteractionTable;
	private EdgeInteractionTableModel edgeInteractionTableModel;
	public boolean okToRender = false;
	
	private Map functionMap;
	Map functionToID;
	Map IDtoFunction;
	private int maxFunctionID;
	private String downstreamDisplayLabel;
	
	private String locAttribute;
	private String saveLocAttribute;
	private String downstreamAttribute;
	private boolean downstreamIsEdgeAttribute;
	private String saveDownstreamAttribute;
	private String functionAttribute;
	private Map locAttributeMap;
	private Map layerNamesMap;
	private Map anyLayerMap;
	private Map layerNumberMap;

	private Map downstreamAttributeMap;
	private Map downstreamMap;
	
	private final JDialog dialog;
	private int restoreIndex;
	
	private final String DELIM = "=";
	public CreateViewDialog() {
		functionToID = new HashMap();
		IDtoFunction = new HashMap();
		maxFunctionID = 0;
		
		dialog = new JDialog ( Cytoscape.getDesktop(),"Create Cerebral view", true);
		
		
	}
	
	public void showDialog(){

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	
		//Add the Layout/cancel buttosn
		final JTextField downstreamText = new JTextField(downstreamDisplayLabel);
		
		JPanel okCancelButtonPanel = new JPanel();
		okCancelButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		okCancelButtonPanel.setLayout( new BoxLayout(okCancelButtonPanel, BoxLayout.X_AXIS));
		okCancelButtonPanel.add(Box.createHorizontalGlue());
		JButton cancel = new JButton ("Cancel");
		cancel.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				okToRender = false;
				dialog.setVisible(false);
			}			
		});
		okCancelButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		okCancelButtonPanel.add(cancel);
		okCancelButtonPanel.add(Box.createRigidArea(new Dimension(5,0)));
		
		final JButton createLayout = new JButton ("Create Layout");
		createLayout.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				okToRender = true;
				downstreamDisplayLabel = downstreamText.getText();				
				extractDataTableInformation();
				dialog.setVisible(false);
			}

				
		});
		okCancelButtonPanel.add(createLayout);
		
		JPanel localizationPanel = new JPanel();
		localizationPanel.setLayout(new BoxLayout(localizationPanel, BoxLayout.Y_AXIS));
		localizationPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),"Localization"));
		localizationPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		add (localizationPanel);
		//Drop-down  attribute selector		
		
		//Load all possible attribute columns
		String[] nodeAttribNames = Cytoscape.getNodeAttributes().getAttributeNames();
		String[] edgeAttribNames = Cytoscape.getEdgeAttributes().getAttributeNames();
		String[] attribNames = new String [nodeAttribNames.length+edgeAttribNames.length + 1];
		final int nodeAttribCount = nodeAttribNames.length;
		System.arraycopy (nodeAttribNames,0, attribNames, 0, nodeAttribNames.length);
		attribNames [nodeAttribCount] = "---------------------------------";
		System.arraycopy (edgeAttribNames, 0, attribNames, nodeAttribCount+1, edgeAttribNames.length);
		
		String[] attribNamesWithBlank = new String[attribNames.length + 1];		
		System.arraycopy(attribNames, 0, attribNamesWithBlank, 1, attribNames.length);
		attribNamesWithBlank[0] = new String("");
		
		String[] nodeAttribNamesWithBlank = new String[nodeAttribCount + 1];
		System.arraycopy(nodeAttribNames, 0, nodeAttribNamesWithBlank,1, nodeAttribCount);
		nodeAttribNamesWithBlank[0] = new String("");
		
		String attribToSearchFor = locAttribute;
		if (locAttribute == null) {
			attribToSearchFor = "localization";
		}
		int initialIndex = 0;
		for (int i=0; i <nodeAttribNames.length; i++){
			if (nodeAttribNames[i].equalsIgnoreCase(attribToSearchFor)){
				initialIndex = i;
				break;
			}
		}
		
		JComboBox attributeBox = new JComboBox(nodeAttribNames);
		attributeBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel chooseAttributeLabel = new JLabel("Select the attribute that contains subcellular localization information:");
		chooseAttributeLabel.setLabelFor(attributeBox);
		localizationPanel.add (chooseAttributeLabel);
		localizationPanel.add(Box.createRigidArea(new Dimension(0,5)));	      
		localizationPanel.add (attributeBox);		
		localizationPanel.add(Box.createRigidArea(new Dimension(0,10)));

		
		//Attribute sorter		

		JLabel orderAttributeLabel = new JLabel("<html>Each subcellular localization will appear as a horizontal layer.  You can adjust the order of these layers using the controls below:", JLabel.LEFT);
		
		JPanel attributeOrderer = new JPanel();		
		attributeOrderer.setAlignmentX(Component.LEFT_ALIGNMENT);	
		orderAttributeLabel.setLabelFor(attributeOrderer);
		localizationPanel.add (orderAttributeLabel);		
		attributeOrderer.setLayout( new BoxLayout(attributeOrderer, BoxLayout.X_AXIS));

		localizationPanel.add(Box.createRigidArea(new Dimension(0,5)));

		localizationPanel.add (attributeOrderer);
		
		attributeOrderTable = new JTable();		
		attributeOrderTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane scrollPane = new JScrollPane(attributeOrderTable);
		scrollPane.setPreferredSize(new Dimension(250, 150));
		attributeOrderer.add (scrollPane);
		
		
		attributeOrderer.add(Box.createRigidArea(new Dimension(5,0)));

		JPanel buttonPanel = new JPanel();
		
		attributeOrderer.add(buttonPanel);
		buttonPanel.setLayout( new BoxLayout(buttonPanel, BoxLayout.Y_AXIS));

		final JButton moveDown = new JButton(new BevelArrowIcon(BevelArrowIcon.DOWN,true, true));
		moveDown.setPressedIcon(new BevelArrowIcon(BevelArrowIcon.DOWN,false, true));
		moveDown.setDisabledIcon(new BevelArrowIcon(BevelArrowIcon.DOWN, true, false));
		moveDown.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				int row =attributeOrderTable.getSelectedRow();
				if (row < attributeOrderTable.getRowCount() - 1) {
					localizationTableModel.moveDown(row);
					attributeOrderTable.setRowSelectionInterval(row+1, row+1);
				}
			}
			
		});

		final JButton moveUp = new JButton(new BevelArrowIcon(BevelArrowIcon.UP,true,true));
		moveUp.setDisabledIcon(new BevelArrowIcon(BevelArrowIcon.UP, true, false));
		moveUp.setPressedIcon(new BevelArrowIcon(BevelArrowIcon.UP, false, true));
	
		moveUp.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				int row =attributeOrderTable.getSelectedRow();
				if (row > 0) {
					localizationTableModel.moveUp(row);
					attributeOrderTable.setRowSelectionInterval(row-1, row-1);
				}
			}			
		});
		moveUp.setSize(moveDown.getSize());
		buttonPanel.add(moveUp);
		buttonPanel.add(moveDown);
		
		ListSelectionModel rowSM = attributeOrderTable.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e) {
		        //Ignore extra messages.
		        if (e.getValueIsAdjusting()) return;

		        ListSelectionModel lsm =
		            (ListSelectionModel)e.getSource();
		        if (lsm.isSelectionEmpty()) {
		           moveUp.setEnabled(false);
		           moveDown.setEnabled(false);
		        } else {
		            int selectedRow = lsm.getMinSelectionIndex();
		            moveUp.setEnabled( selectedRow >= 1);
		            moveDown.setEnabled( selectedRow <= attributeOrderTable.getRowCount() -2);
		        }
		    }

		});
		
		attributeBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox) e.getSource();
				locAttribute = (String)cb.getSelectedItem();								
				localizationTableModel = new LocalizationTableModel(locAttribute);
				attributeOrderTable.setModel ( localizationTableModel);
				TableColumn column = null;		
				column = attributeOrderTable.getColumnModel().getColumn(0);
				column.setPreferredWidth(40);
				column = attributeOrderTable.getColumnModel().getColumn(1);
				column.setPreferredWidth(150);
				column = attributeOrderTable.getColumnModel().getColumn(2);
				column.setPreferredWidth(200);
		
			}
		
		});		
		attributeBox.setSelectedIndex(initialIndex);

		//Downstream interaction chooser
		
		JPanel downstreamPanel = new JPanel();
		downstreamPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		add(Box.createRigidArea(new Dimension(0,20)));
		add (downstreamPanel);
		downstreamPanel.setLayout(new BoxLayout(downstreamPanel, BoxLayout.Y_AXIS));
		downstreamPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(),"Downstream nodes"));
		//See if there's an attribute called interaction
		attribToSearchFor = downstreamAttribute;
		if (attribToSearchFor == null){
			attribToSearchFor = "interaction";
		}
				
		initialIndex = 0;
		for (int i=0; i <attribNamesWithBlank.length; i++){
			if (attribNamesWithBlank[i].equalsIgnoreCase(attribToSearchFor)){
				initialIndex = i;
				break;
			}
		}
		
		JComboBox edgeAttributeBox = new JComboBox(attribNamesWithBlank);
		edgeAttributeBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel downstreamLabel = new JLabel ("<html>You may wish to display certain nodes in an additional layer below all other nodes, for example, genes downstream of a particular pathway.", JLabel.LEFT);
		JLabel downstreamLabel2 = new JLabel("<html>	Select the attribute specifying downstream nodes:", JLabel.LEFT);
		downstreamLabel.setLabelFor(edgeAttributeBox);
		downstreamLabel2.setLabelFor(edgeAttributeBox);
		downstreamLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		downstreamLabel2.setAlignmentX(Component.LEFT_ALIGNMENT);
		downstreamPanel.add (downstreamLabel);
		downstreamPanel.add (downstreamLabel2);
		downstreamPanel.add(Box.createRigidArea(new Dimension(0,5	)));

		
		downstreamPanel.add (edgeAttributeBox);
		
		edgeAttributeBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox) e.getSource();
				downstreamAttribute = (String)cb.getSelectedItem();
				edgeInteractionTableModel = new EdgeInteractionTableModel(downstreamAttribute, cb.getSelectedIndex() > nodeAttribCount);
				edgeInteractionTable.setModel (edgeInteractionTableModel);
				TableColumn column = null;		
				column = edgeInteractionTable.getColumnModel().getColumn(0);
				column.setPreferredWidth(40);
				column = edgeInteractionTable.getColumnModel().getColumn(1);
				column.setPreferredWidth(150);			}		
		});		

		
		edgeInteractionTable = new JTable();
		edgeInteractionTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scrollPane = new JScrollPane(edgeInteractionTable);	
		scrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		scrollPane.setPreferredSize(new Dimension(250, 150));		
		downstreamPanel.add(Box.createRigidArea(new Dimension(0,10)));

		downstreamPanel.add (scrollPane);
		edgeAttributeBox.setSelectedIndex(initialIndex);
		
		JLabel downstreamInputLabel = new JLabel("<html>Provide a label for the extra layer, e.g. Downstream genes: ");
		JPanel downstreamInput = new JPanel();
		downstreamInput.setLayout(new BoxLayout(downstreamInput, BoxLayout.X_AXIS));
		downstreamInput.setAlignmentX(Component.LEFT_ALIGNMENT);		
		downstreamInputLabel.setLabelFor(downstreamText);
		downstreamInput.add(downstreamInputLabel);
		downstreamInput.add(downstreamText);	
		downstreamPanel.add(Box.createRigidArea(new Dimension(0,10)));
		downstreamPanel.add(downstreamInput);
		
	
//		Function grouping
		
		JComboBox functionAttributeBox = new JComboBox(nodeAttribNamesWithBlank);		
		functionAttributeBox.addActionListener( new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JComboBox cb = (JComboBox) e.getSource();
				functionAttribute = (String)cb.getSelectedItem();
				functionMap = CyAttributesUtils.getAttribute(functionAttribute,Cytoscape.getNodeAttributes());			
			}
		
		});		
		functionAttributeBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel functionLabel = new JLabel("<html>These nodes can be further clustered by an attribute.  For example, if functional information is loaded, downstream nodes can be grouped by function.", JLabel.LEFT);
		JLabel functionLabel2= new JLabel("<html>Select the attribute that will be used to cluster downstream genes:", JLabel.LEFT);
		functionLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		functionLabel2.setAlignmentX(Component.LEFT_ALIGNMENT);
		downstreamPanel.add(Box.createRigidArea(new Dimension(0,10)));		
		downstreamPanel.add (functionLabel);
		downstreamPanel.add (functionLabel2);
		functionLabel.setLabelFor(functionAttributeBox);
		functionLabel2.setLabelFor(functionAttributeBox);
		downstreamPanel.add(Box.createRigidArea(new Dimension(0,10)));
		
		downstreamPanel.add (functionAttributeBox);
		
		attribToSearchFor = functionAttribute;
		if (attribToSearchFor == null) {
			attribToSearchFor = "function";
		}
		initialIndex = 0;
		for (int i=0; i <nodeAttribNamesWithBlank.length; i++){			
			if (nodeAttribNamesWithBlank[i].equalsIgnoreCase(attribToSearchFor)){
				initialIndex = i;
				break;
			}
		}
		functionAttributeBox.setSelectedIndex(initialIndex);
		
		add (Box.createRigidArea(new Dimension(0,20)));
		add(okCancelButtonPanel);
		
		
		
		dialog.getContentPane().add(this);
		dialog.pack();
		dialog.setLocationRelativeTo(Cytoscape.getDesktop());
		getRootPane().setDefaultButton(createLayout);
		dialog.setVisible(true);
			
	}
	
	private class LocalizationTableModel extends AbstractTableModel{
	
		private static final long serialVersionUID = 1L;

		String[] localizationColumns = {
				"Layer",
				"Localization",
				  "Allow placement in any layer"
		};
		
		private String[] layer;
		private String[] attributes;
		private Boolean[] anyLayer;
		
		private Map attributeMap;
		
		public LocalizationTableModel (String attribute){
			attributeMap = CyAttributesUtils.getAttribute((attribute),
					Cytoscape.getNodeAttributes());
	
			Set uniqueAttributes = new HashSet();
			if (attributeMap != null){
				Iterator attributeIt = attributeMap.values().iterator();
				while (attributeIt.hasNext()){
					uniqueAttributes.add(attributeIt.next().toString());			
				}
			}
			
			int numAttributes = uniqueAttributes.size();
			
			layer = new String[numAttributes];	
			attributes = new String[numAttributes];
			anyLayer = new Boolean[numAttributes];
		    int i=0;
		    
		    if (attribute.equalsIgnoreCase(saveLocAttribute) && layerNumberMap != null){
		    	//we've previously used this attribute.
		    	//restore the ordering
		    	Iterator keys = layerNumberMap.keySet().iterator();
		    	while (keys.hasNext()) {
		    		String key = (String)keys.next();
		    		Integer layer = (Integer)layerNumberMap.get(key);
		    		if (layer.intValue() < numAttributes){
		    			attributes[layer.intValue()] = key;
		    			uniqueAttributes.remove(key);
		    			if (layer.intValue()  > i ) {
		    				i = layer.intValue();
		    			}
		    		}
		    	}
		    	i++;
		    }else {
			    //Try for our default ordering.  Unlikely to 
			    //succeed except in demos
			    if (uniqueAttributes.contains("Extracellular")){
			      	attributes[i++] = "Extracellular";
			      	uniqueAttributes.remove("Extracellular");
			    }else if (uniqueAttributes.contains("extracellular")){
			     	attributes[i++] = "extracellular";
			      	uniqueAttributes.remove("extracellular");
			    }
			    
			    if (uniqueAttributes.contains("Plasma membrane")){
			      	attributes[i++] = "Plasma membrane";
			      	uniqueAttributes.remove("Plasma membrane");
			    }else if (uniqueAttributes.contains("plasma membrane")){
			     	attributes[i++] = "plasma membrane";
			      	uniqueAttributes.remove("plasma membrane");
			    }
			    if (uniqueAttributes.contains("Cytoplasm")){
			      	attributes[i++] = "Cytoplasm";
			      	uniqueAttributes.remove("Cytoplasm");
			    }else if (uniqueAttributes.contains("cytoplasm")){
			     	attributes[i++] = "cytoplasm";
			      	uniqueAttributes.remove("cytoplasm");
			    }
			    
			    if (uniqueAttributes.contains("Nucleus")){
			      	attributes[i++] = "Nucleus";
			      	uniqueAttributes.remove("Nucleus");
			    }else if (uniqueAttributes.contains("nucleus")){
			     	attributes[i++] = "nucleus";
			      	uniqueAttributes.remove("nucleus");
			    }
			 }
			    
			Iterator attributeIt = uniqueAttributes.iterator();			
		 	while (attributeIt.hasNext()){
				attributes[i++] =  attributeIt.next().toString();
			}
		 	
		 	if (attribute.equalsIgnoreCase(saveLocAttribute) && anyLayerMap != null){
				for ( i=0; i < numAttributes; i++){
					if (anyLayerMap.containsKey (attributes[i])){
						anyLayer[i] = (Boolean) anyLayerMap.get(attributes[i]);
					}else{						
						anyLayer[i] = new Boolean(false);
					}
				}		
		 	}else {			   
			
				for ( i=0; i < numAttributes; i++){
					boolean b = false;
					if (attributes[i].compareTo("") ==0){
						b = true;
					}else if (attributes[i].compareToIgnoreCase("unknown")==0){
						b = true;
					}
					anyLayer[i] = new Boolean(b);
				}
		 	}
			assignLayers();
		}
		
		private void assignLayers(){

			  int layerNumber =1;
		      for (int i=0; i < layer.length; i++){
		    	  if (anyLayer[i].booleanValue()){
		    		  layer[i] = new String("");
		    	  }else {
		    		  layer[i] = Integer.toString(layerNumber++);
		    	  }
		    	  fireTableCellUpdated(i, 0);					
		      }
				
		}
		
		public void moveUp(int row) {
				String saveAttributes = attributes[row-1];
				Boolean saveAnyLayer = anyLayer[row -1];
				setValueAt(attributes[row], row-1,1);
				setValueAt(anyLayer[row],row-1, 2);
				setValueAt(saveAttributes, row,1);
				setValueAt (saveAnyLayer,row, 2);				
		}
		public void moveDown(int row) {
				String saveAttributes = attributes[row+1];
				Boolean saveAnyLayer = anyLayer[row +1];
				setValueAt(attributes[row], row+1,1);
				setValueAt(anyLayer[row],row+1, 2);
				setValueAt(saveAttributes, row,1);
				setValueAt (saveAnyLayer,row, 2);							
		}
		
		public int getRowCount() {
			return layer.length;
		}

		public int getColumnCount() {
			return localizationColumns.length;
		}
		
		public String getColumnName(int col) {
		        return localizationColumns[col];
		 }
		public Object getValueAt(int r, int c) {
			switch (c){
			case 1:
				return attributes[r];
			case 0:				
				return layer[r];
			case 2:
				return anyLayer[r];
			}
			return null;
		}
		public Class getColumnClass(int c) {
			switch (c){
			case 1:
				return String.class;
			case 0:				
				return String.class;
			case 2:
				return Boolean.class;
			}
			return Object.class;
		}
		
		  public boolean isCellEditable(int row, int col) {
			  return col == 2;
		   }
		  
		  public void setValueAt(Object value, int row, int col) {
			 switch (col) {
			 case 0:
				 layer[row] = (String) value; break;
			 case 1:
				 attributes[row] = (String) value; break;
			 case 2:
			 	  anyLayer[row] = (Boolean)value; break;
			  }	  
			  fireTableCellUpdated(row, col);
		      assignLayers();
		 }

		public Map getLayerNameMap() {
			Map returnMap = new HashMap();
		    for (int i=0; i < layer.length; i++){
		      if (!layer[i].equals("")){
	    		  returnMap.put(new Integer(
	    				  Integer.parseInt(layer[i])-1), attributes[i]);
		    	  
			  }
		    }
		    return returnMap;
		}

		public Map getAnyLayerMap() {
			Map returnMap = new HashMap();
			for (int i = 0; i < attributes.length; i++){
				returnMap.put(attributes[i], anyLayer[i]);
			}
			return returnMap;
		}

		public Map getLayerNumberMap() {
			Map returnMap = new HashMap();
			for (int i = 0; i < attributes.length; i++){
				if (!layer[i].equals("")){
					returnMap.put(attributes[i], 
						new Integer( Integer.parseInt(layer[i])-1));
				}
				
			}
			return returnMap;
		}

	}
	
	
	private class EdgeInteractionTableModel extends AbstractTableModel{
		
		private static final long serialVersionUID = 1L;

		String[] localizationColumns;
		
		private String[] attributes;
		private Boolean[] activated;
		private Map attributeMap;
		private boolean isEdgeAttribute;
		public EdgeInteractionTableModel (String attribute, boolean isEdgeAttribute){
			this.isEdgeAttribute = isEdgeAttribute;
			if (isEdgeAttribute){
				attributeMap = CyAttributesUtils.getAttribute(attribute, Cytoscape.getEdgeAttributes());
			}else{
				attributeMap = CyAttributesUtils.getAttribute((attribute),
					Cytoscape.getNodeAttributes());
			}
			localizationColumns = new String[2];
			localizationColumns[0] = attribute;
			localizationColumns[1] = "Downstream node";
			Set uniqueAttributes = new HashSet();
			if (attributeMap != null){
				Iterator attributeIt = attributeMap.values().iterator();
				while (attributeIt.hasNext()){
					uniqueAttributes.add(attributeIt.next().toString());			
				}
			}
	
			attributes = new String[uniqueAttributes.size()];
			Iterator attributeIt = uniqueAttributes.iterator();
		    int i=0;
			while (attributeIt.hasNext()){
				attributes[i++] = attributeIt.next().toString();
			}
			
			activated = new Boolean[uniqueAttributes.size()];
			
			if (attribute.equals(saveDownstreamAttribute) && downstreamMap != null){
				for (i=0; i < attributes.length; i++){				
					if (downstreamMap.containsKey(attributes[i])){
						activated[i] = (Boolean) downstreamMap.get(attributes[i]);
					}else{							
						activated[i] = new Boolean(false);
					}
				}
			}else{
				for (i=0; i < attributes.length; i++){				
					activated[i] = new Boolean(
								attributes[i].compareToIgnoreCase("pd")==0
					);
				}
			}
		}
		public int getRowCount() {
			return attributes.length;
		}

		public int getColumnCount() {
			return localizationColumns.length;
		}
		
		public String getColumnName(int col) {
		        return localizationColumns[col];
		 }
		public Object getValueAt(int r, int c) {
			switch (c){
			case 0:
				return attributes[r];
			case 1:
				return activated[r];
			}
			return null;
		}
		public Class getColumnClass(int c) {
			switch (c){
			case 0:				
				return String.class;
			case 1:
				return Boolean.class;
			}
			return Object.class;
		}
		
		  public boolean isCellEditable(int row, int col) {
			  return col == 1;
		   }
		  
		  public void setValueAt(Object value, int row, int col) {
			 switch (col) {
			 case 1:
			 	  activated[row] = (Boolean)value; break;
			  }	  
			  fireTableCellUpdated(row, col);		   
		 }
		  
		
		public Map getDownstreamMap() {
			Map returnMap = new HashMap();
		    for (int i=0; i < attributes.length; i++){
		    	returnMap.put(attributes[i], activated[i]);
		    }		     
		    return returnMap;
		}

	}

	private void extractDataTableInformation() {
		layerNamesMap = localizationTableModel.getLayerNameMap();
		anyLayerMap = localizationTableModel.getAnyLayerMap();
		layerNumberMap = localizationTableModel.getLayerNumberMap();
		locAttributeMap = localizationTableModel.attributeMap;
		
		downstreamAttributeMap = edgeInteractionTableModel.attributeMap;
		downstreamMap = edgeInteractionTableModel.getDownstreamMap();
		downstreamIsEdgeAttribute = edgeInteractionTableModel.isEdgeAttribute;
	}		
	public int getNumLayers() {
		return layerNumberMap.size();
	}

	public String getLayerName(int i) {
		return (String) layerNamesMap.get(new Integer(i));		
	}


	public boolean placeAnyLayer(Node node) {
		if (locAttributeMap == null){
			return true;
		}else{
			Object objBioCategory =  locAttributeMap.get(node.getIdentifier());
			if (objBioCategory == null)
				return true;
			String bioCategory = objBioCategory.toString();
			if (!anyLayerMap.containsKey(bioCategory)){
				return true;
			}else{
				return ((Boolean)anyLayerMap.get(bioCategory)).booleanValue();
			}
		}
	}


	public int getLayer(Node node) {
		String bioCategory = locAttributeMap.get(node.getIdentifier()).toString();
		return ((Integer)layerNumberMap.get(bioCategory)).intValue();
	}


	public boolean bioFunctionUnknown(Node node) {
		if (functionMap == null){
			return true;
		}else {
			Object objBioFunction =  functionMap.get(node.getIdentifier());
			if (objBioFunction == null)
				return true;			
			if ((objBioFunction instanceof String))	{
				String strBioFunction = objBioFunction.toString();
				return strBioFunction.trim().length() == 0;
			}else{ 
				return false;
			}
		}		
	}


	public int getFunction(Node node) {
		Object objFunction = functionMap.get(node.getIdentifier());
		String function =  objFunction.toString();
		if (functionToID.containsKey(function)) {
			return ((Integer) functionToID.get(function)).intValue();
		} else {
			functionToID.put(function, new Integer(maxFunctionID));
			IDtoFunction.put(new Integer(maxFunctionID), function);
			maxFunctionID++;
			return maxFunctionID -1;
		}
	
	}


	public int getNumFunctions() {
		return maxFunctionID;
	}


	public String getFunctionName(int i) {	
		return (String) IDtoFunction.get(new Integer(i));
	}


	public boolean isDownstreamGene(String identifier) {
		if (downstreamAttributeMap == null){
			return false;
		}else{
			Object objAttribute = downstreamAttributeMap.get(identifier);
			if (objAttribute == null)
				return false;
			String attribute = objAttribute.toString();
			if (!downstreamMap.containsKey(attribute)){
				return false;
			}else{
				return ((Boolean) downstreamMap.get(attribute)).booleanValue();
			}
		}
	}
	public boolean isDownstreamGene(Edge edge) {
		return isDownstreamGene(edge.getIdentifier());
	}
	
	public boolean isDownstreamGene(Node node) {
		return isDownstreamGene(node.getIdentifier());
	}

	public boolean downstreamIsEdgeAttribute(){
		return downstreamIsEdgeAttribute;
	}
	
	public boolean displayDownstreamGenes(){
		return downstreamAttributeMap != null;
	}

	public String getDownstreamLabel(){
		return downstreamDisplayLabel;
	}

	public String getSaveString() {
		String saveString = "";
		saveString += "version1.1";
		saveString += DELIM;
		saveString += downstreamDisplayLabel;
		saveString += DELIM;
		saveString += locAttribute;
		saveString += DELIM;
		saveString += downstreamAttribute;
		saveString += DELIM;		
		saveString += downstreamIsEdgeAttribute;
		saveString += DELIM;
		saveString += functionAttribute;
		saveString += DELIM;
		saveString += saveMap (layerNamesMap);
		saveString += saveMap (anyLayerMap);
		saveString += saveMap (layerNumberMap);
		saveString += saveMap (downstreamMap);
		
		return saveString;
	}

	private String saveMap(Map map) {
		String saveString = "";
		Iterator keysIt = map.keySet().iterator();
		while (keysIt.hasNext()){
			Object key = keysIt.next();
			Object value = map.get(key);
			saveString += key.toString();
			saveString += DELIM;
			saveString += value.toString();
			saveString += DELIM;			
		}
		saveString += DELIM;
		return saveString;
	}
	
	public void loadFromString (String saveString){

		layerNamesMap = new HashMap();
		layerNumberMap = new HashMap();
		downstreamMap = new HashMap();		
		anyLayerMap = new HashMap();
	   restoreIndex = 0;
	   String nextVal = getNextValue(saveString);
		if (!nextVal.equals("version1.1")){
			System.out.println ("wrong version.");
			return;
		}
		downstreamDisplayLabel = getNextValue( saveString);
        if (downstreamDisplayLabel== null)
            downstreamDisplayLabel = "";
		locAttribute = getNextValue(saveString);
		saveLocAttribute = locAttribute;
		locAttributeMap = CyAttributesUtils.getAttribute((locAttribute), 	Cytoscape.getNodeAttributes());
		downstreamAttribute = getNextValue (saveString);
		saveDownstreamAttribute = downstreamAttribute;
		downstreamIsEdgeAttribute = new Boolean(getNextValue(saveString)).booleanValue();
		if (downstreamIsEdgeAttribute)
			downstreamAttributeMap= CyAttributesUtils.getAttribute((downstreamAttribute),	Cytoscape.getEdgeAttributes());
		else
			downstreamAttributeMap= CyAttributesUtils.getAttribute((downstreamAttribute),	Cytoscape.getNodeAttributes());
		
		functionAttribute = getNextValue(saveString);
		functionMap = CyAttributesUtils.getAttribute(functionAttribute,Cytoscape.getNodeAttributes());					
		restoreIntegerStringMap(layerNamesMap, saveString);
		restoreStringBooleanMap(anyLayerMap, saveString);
		restoreStringIntegerMap(layerNumberMap, saveString);
		restoreStringBooleanMap(downstreamMap, saveString);
		
		okToRender = true;
	}

	private String getNextValue(String saveString) {
		int endIndex =saveString.indexOf(DELIM, restoreIndex);
		String value = saveString.substring(restoreIndex, endIndex);
		restoreIndex = endIndex +1;
		return value;
	}
	
	private void restoreIntegerStringMap (Map map, String saveString){
		String nextVal = getNextValue(saveString);
		while (!nextVal.equals("")){
			String key = nextVal;
			String value = getNextValue(saveString);
			map.put(new Integer(key), value);
			nextVal = getNextValue(saveString);
		}
	}
	
	private void restoreStringBooleanMap (Map map, String saveString){
		String nextVal = getNextValue(saveString);
		while (!nextVal.equals("")){
			String key = nextVal;
			String value = getNextValue(saveString);
			map.put(key, new Boolean(value));
			nextVal = getNextValue(saveString);
		}
	}
	private void restoreStringIntegerMap (Map map, String saveString){
		String nextVal = getNextValue(saveString);
		while (!nextVal.equals("")){
			String key = nextVal;
			String value = getNextValue(saveString);
			map.put(key, new Integer(value));
			nextVal = getNextValue(saveString);
		}
	}

	
	
}
